@@ -516,6 +516,9 @@ void ArticleChecker::detect_redundant_articles()
 
 class TaskStream
 {
+public: // constants
+    const static size_t MAX_SIZE = 1000;
+
 public: // functions
     explicit TaskStream(ArticleChecker* ac)
         : articleChecker(*ac)
@@ -542,16 +545,20 @@ public: // functions
     void addTask(zim::Entry entry)
     {
         assert(expectingMoreTasks);
-        std::lock_guard<std::mutex> lock(mutex);
+        std::unique_lock<std::mutex> lock(mutex);
+
+        while ( inIsBlocked() )
+            waitUntilInIsUnblocked(lock);
+
         taskQueue.push(entry);
-        unblock();
+        unblockOut();
     }
 
     void noMoreTasks()
     {
         std::lock_guard<std::mutex> lock(mutex);
         expectingMoreTasks = false;
-        unblock();
+        unblockOut();
     }
 
 private: // types
@@ -569,17 +576,32 @@ private: // functions
         }
     }
 
-    bool blocked() const
+    bool inIsBlocked() const
+    {
+        return taskQueue.size() >= MAX_SIZE;
+    }
+
+    bool outIsBlocked() const
     {
         return expectingMoreTasks && taskQueue.empty();
     }
 
-    void waitUntilUnblocked(std::unique_lock<std::mutex>& lock)
+    void waitUntilInIsUnblocked(std::unique_lock<std::mutex>& lock)
     {
         cv.wait(lock);
     }
 
-    void unblock()
+    void waitUntilOutIsUnblocked(std::unique_lock<std::mutex>& lock)
+    {
+        cv.wait(lock);
+    }
+
+    void unblockIn()
+    {
+        cv.notify_one();
+    }
+
+    void unblockOut()
     {
         cv.notify_one();
     }
@@ -587,14 +609,15 @@ private: // functions
     Task getNextTask()
     {
         std::unique_lock<std::mutex> lock(mutex);
-        if ( blocked() )
-            waitUntilUnblocked(lock);
+        if ( outIsBlocked() )
+            waitUntilOutIsUnblocked(lock);
 
         Task t;
         if ( !taskQueue.empty() )
         {
             t = std::make_shared<zim::Entry>(taskQueue.front());
             taskQueue.pop();
+            unblockIn();
         }
         return t;
     }
@@ -604,7 +627,18 @@ private: // data
     std::queue<zim::Entry> taskQueue;
     std::mutex mutex;
     std::thread thread;
+
+    // cv is used for managing the blocking & unblocking of both ends of the
+    // task queue. This is possible with a single std::condition_variable due
+    // to the fact that both ends of the queue cannot be blocked at the same
+    // time:
+    // 1. when the input is blocked (because the queue is full), the consumer
+    //    is free to proceed (thus unblocking the input).
+    // 2. when the output is blocked (becuase the queue is empty while more data
+    //    has been promised) the producer is welcome to use the input end.
+    // Thus only the consumer or the producer waits on cv.
     std::condition_variable cv;
+
     std::atomic<bool> expectingMoreTasks;
 };
 
@@ -668,22 +702,106 @@ void test_articles(const zim::Archive& archive, ErrorLogger& reporter, ProgressB
     }
 }
 
+namespace
+{
+
+class RedirectionTable
+{
+private: // types
+    enum LoopStatus : uint8_t
+    {
+        UNKNOWN,
+        LOOP,
+        NONLOOP
+    };
+
+public: // functions
+    explicit RedirectionTable(size_t entryCount)
+    {
+        loopStatus.reserve(entryCount);
+        redirTable.reserve(entryCount);
+    }
+
+    void addRedirectionEntry(zim::entry_index_type targetEntryIndex)
+    {
+        redirTable.push_back(targetEntryIndex);
+        loopStatus.push_back(LoopStatus::UNKNOWN);
+    }
+
+    void addItem()
+    {
+        redirTable.push_back(redirTable.size());
+        loopStatus.push_back(LoopStatus::NONLOOP);
+    }
+
+    size_t size() const { return redirTable.size(); }
+
+    bool isInRedirectionLoop(zim::entry_index_type i)
+    {
+        if ( loopStatus[i] == UNKNOWN )
+        {
+            resolveLoopStatus(i);
+        }
+
+        return loopStatus[i] == LOOP;
+    }
+
+private: // functions
+    LoopStatus detectLoopStatus(zim::entry_index_type i) const
+    {
+        auto i1 = i;
+        auto i2 = i;
+        // Follow redirections until an entry with known loop status
+        // is found.
+        // i2 moves through redirections at twice the speed of i1
+        // if i2 runs over i1 then they are both inside a redirection loop
+        for (bool moveI1 = false ; ; moveI1 = !moveI1)
+        {
+            if ( loopStatus[i2] != LoopStatus::UNKNOWN )
+                return loopStatus[i2];
+
+            i2 = redirTable[i2];
+
+            if ( i2 == i1 )
+                return LoopStatus::LOOP;
+
+            if ( moveI1 )
+                i1 = redirTable[i1];
+        }
+    }
+
+    void resolveLoopStatus(zim::entry_index_type i)
+    {
+        const LoopStatus s = detectLoopStatus(i);
+        for ( ; loopStatus[i] == LoopStatus::UNKNOWN; i = redirTable[i] )
+        {
+            loopStatus[i] = s;
+        }
+    }
+
+private: // data
+    std::vector<zim::entry_index_type> redirTable;
+    std::vector<LoopStatus> loopStatus;
+};
+
+} // unnamed namespace
+
 void test_redirect_loop(const zim::Archive& archive, ErrorLogger& reporter) {
     reporter.infoMsg("[INFO] Checking for redirect loops...");
 
-    int chained_redirection_limit = 50;
-
-    for(auto& entry: archive.iterEfficient())
+    RedirectionTable redirTable(archive.getAllEntryCount());
+    for(const auto& entry: archive.iterByPath())
     {
-        auto current_entry = entry;
-        int redirections_done = 0;
-        while(current_entry.isRedirect() && redirections_done < chained_redirection_limit)
-        {
-            current_entry = current_entry.getRedirectEntry();
-            redirections_done++;
-        }
+        if ( entry.isRedirect() )
+            redirTable.addRedirectionEntry(entry.getRedirectEntryIndex());
+        else
+            redirTable.addItem();
+    }
 
-        if(current_entry.isRedirect()){
+    for(zim::entry_index_type i = 0; i < redirTable.size(); ++i )
+    {
+        if(redirTable.isInRedirectionLoop(i)){
+            const auto entry = archive.getEntryByPath(i);
             reporter.addMsg(MsgId::REDIRECT_LOOP, {{"entry_path", entry.getPath()}});
         }
     }
