@@ -639,7 +639,9 @@ TEST(zimcheck, external_url_check_poorzimfile)
       "[INFO] Zimcheck version is " VERSION "\n"
       "[INFO] Verifying Articles' content..." "\n"
       "[ERROR] Invalid external links found:" "\n"
-      "  http://a.io/pic.png is an external dependence in article external_link.html" "\n"
+      "  http://a.io/pic.png is an external dependence in article external_image_http.html" "\n"
+      "  https://a.io/pic.png is an external dependence in article external_image_https.html" "\n"
+      "  //a.io/pic.png is an external dependence in article external_image_protocol_relative.html" "\n"
       "[INFO] Overall Test Status: Fail" "\n"
       "[INFO] Total time taken by zimcheck: <3 seconds." "\n"
     );
@@ -732,7 +734,9 @@ const std::string ALL_CHECKS_OUTPUT_ON_POORZIMFILE(
       "  Found 1 empty links in article: empty_link.html" "\n"
       "  ../../oops.html is out of bounds. Article: outofbounds_link.html" "\n"
       "[ERROR] Invalid external links found:" "\n"
-      "  http://a.io/pic.png is an external dependence in article external_link.html" "\n"
+      "  http://a.io/pic.png is an external dependence in article external_image_http.html" "\n"
+      "  https://a.io/pic.png is an external dependence in article external_image_https.html" "\n"
+      "  //a.io/pic.png is an external dependence in article external_image_protocol_relative.html" "\n"
       "[ERROR] Redirect loop(s) exist:" "\n"
       "  Redirect loop exists from entry redirect_loop.html" "\n"
       "" "\n"
@@ -882,9 +886,23 @@ TEST(zimcheck, json_poorzimfile)
       "    {"                                                               "\n"
       "      \"check\" : \"url_external\","                                 "\n"
       "      \"level\" : \"ERROR\","                                        "\n"
-      "      \"message\" : \"http://a.io/pic.png is an external dependence in article external_link.html\"," "\n"
+      "      \"message\" : \"http://a.io/pic.png is an external dependence in article external_image_http.html\"," "\n"
       "      \"link\" : \"http://a.io/pic.png\","                           "\n"
-      "      \"path\" : \"external_link.html\""                             "\n"
+      "      \"path\" : \"external_image_http.html\""                             "\n"
+      "    },"                                                              "\n"
+      "    {"                                                               "\n"
+      "      \"check\" : \"url_external\","                                 "\n"
+      "      \"level\" : \"ERROR\","                                        "\n"
+      "      \"message\" : \"https://a.io/pic.png is an external dependence in article external_image_https.html\"," "\n"
+      "      \"link\" : \"https://a.io/pic.png\","                           "\n"
+      "      \"path\" : \"external_image_https.html\""                             "\n"
+      "    },"                                                              "\n"
+      "    {"                                                               "\n"
+      "      \"check\" : \"url_external\","                                 "\n"
+      "      \"level\" : \"ERROR\","                                        "\n"
+      "      \"message\" : \"//a.io/pic.png is an external dependence in article external_image_protocol_relative.html\"," "\n"
+      "      \"link\" : \"//a.io/pic.png\","                           "\n"
+      "      \"path\" : \"external_image_protocol_relative.html\""                             "\n"
       "    },"                                                              "\n"
       "    {"                                                               "\n"
       "      \"check\" : \"redirect\","                                     "\n"
